/*
* PhoneyTV by Jim  >> BulldogLowell@gmail.com
* Inspired by Josh >> Deltanu1142@gmail.com
*
* This Sketch illuminates 6 sets of LED's in a random fashion as to mimic the 
* ambient light eminating from a television.  It is intended to make an empty
* home (or an empty section of a home) appear to be occupied by someone watching
* TV.  As an alternative to a real television left on, this uses less than 1%
* of the electrical energy.
*
* With the use of the MySensors plugin and gateway, PhoneyTV is intended to 
* be used with Vera or Raspberry PI.
*
* Sketch does not use any delays to create the random blinking as a way to 
* assure that communication back to the gateway is as unaffected as possible.
*
* You can adjust the length of the blink interval and its "twitchyness" by
* modifying the random number generators, if you prefer more/less 'motion' in
* in your unit.  The lines are highlighted in the code.
*
* Sketch takes advantage of available PWM on pins 3, 5 & 6 using the wht/blu LEDs 
* to allow fluctuations in the intensity of the light, enhancing the PhoneyTV's
* natural ambient light effect.
*
* This uses the MySensors.org sketch and will attach itself to your gateway as
* a light, which was another requirement.  Because of that requirement, it 
* utilizes one digital output pin in the logic (but not physically) and a relay
* is not actually required to use this sketch.  Also, the radio pinout remains
* in tact as to preserve the ease of use for MySensor.org users
*
* Created 12-APR-2014
* Free for distrubution
* Credit should be given to MySensors.org for their base code for relay control
* and for the radio configuration.  Thanks Guys.
*/ 
////
#include <Relay.h>
#include <SPI.h>
#include <EEPROM.h> 
#include <RF24.h>
//
#define RELAY_1  3  // Arduino Digital I/O pin number for the 'relay' yes there is a conflict but it is no issue.
#define NUMBER_OF_RELAYS 1
#define RELAY_ON 0
#define RELAY_OFF 1
//
// set pin numbers for the LEDs
int ledPin3 =  3;      // White using PWM
int ledPin4 =  4;      // Red
int ledPin5 =  5;      // Blue using PWM
int ledPin6 =  6;      // Blue using PWM
int ledPin7 =  7;      // Green
int ledPin8 =  8;      // White (No PWM)
//
unsigned long currentMillis;
int ledState = LOW;             // ledState
long previousMillis = 0;        // will store last time LED was updated
int led = 5;
int elapsedMillis;
int interval = 2000;  // initial interval at which to blink (milliseconds)
int twitch = 50; //initial twitch factor
int dipCount = 5; //dims all of the leds simultaneously, at random counts
int dipTimes;
int analogLevel = 100;
//
int incomingRelayStatus; //global variable to pass relay status in/out of functions
//
Sensor gw;
//
void setup() 
{ 
  //the LED bits
  // set the digital pin as output:
  pinMode(ledPin3, OUTPUT);
  pinMode(ledPin4, OUTPUT); 
  pinMode(ledPin5, OUTPUT);
  pinMode(ledPin6, OUTPUT);
  pinMode(ledPin7, OUTPUT);
  pinMode(ledPin8, OUTPUT);
  Serial.begin(115200);  
  //
  //
  gw.begin();
  // Send the sketch version information to the gateway and Controller
  gw.sendSketchInfo("PhoneyTV", "1.0");
  // Register all sensors to gw (they will be created as child devices)
  for (int i=0; i<NUMBER_OF_RELAYS;i++) {
    gw.sendSensorPresentation(RELAY_1+i, S_LIGHT);
  }
  // Fetch relay status
  for (int i=0; i<NUMBER_OF_RELAYS;i++) {
    // Make sure relays are off when starting up
    digitalWrite(RELAY_1+i, RELAY_OFF);
    // Then set relay pins in output mode
    pinMode(RELAY_1+i, OUTPUT);  
    // Request/wait for relay status
    gw.getStatus(RELAY_1+i, V_LIGHT);
    setRelayStatus(gw.getMessage()); // Wait here until status message arrive from gw
  }
}
//
void loop()
{
  while (incomingRelayStatus==1)
  {
  //Serial.println("in While loop 1");
  getMessage();
  // 'randomly' blink the LEDs.
  currentMillis = millis();
  elapsedMillis = (currentMillis-previousMillis);
  //Serial.println(elapsedMillis);
  if(elapsedMillis > interval) {
    previousMillis = currentMillis;
   interval = random(750,4001);//Adjusts the interval for more random light changes
   twitch = random(40,100);// Twitch provides motion effect but can be a bit much if too high
   dipCount = dipCount+1;
   Serial.print("Interval = ");
   Serial.println(interval);
   Serial.print("Twitchiness = ");
   Serial.println(twitch);
  }
  if(elapsedMillis<twitch){ //use this line to control twitchiness
     led=random(3,9);
     analogLevel=random(50,255);// set the range of the 3 pwm leds
    // if the LED is off turn it on and vice-versa:
    if (ledState == LOW)
      ledState = HIGH;
    else 
      ledState = LOW;
    switch (led) {
      case 3:
      pwmWrite();
      break;
      case 5:
      pwmWrite();
      break;
      case 6:
      pwmWrite();
      break;
      default:
      digitalWrite(led, ledState);
    }
     if (dipCount > dipTimes) {
       dipCall();
     }
   } 
  }//end of while 1 loop
  //
  while (incomingRelayStatus==0)
  {
  for (int i=3;i<9;i++){
  digitalWrite(i,LOW);
  }
    getMessage();
    Serial.println("in zero loop");
  }//end of while zero loop
}
//
void dipCall(){
  for (int i=3; i<9; i++){
    digitalWrite(i, LOW);
    dipTimes = random(10,75);
    dipCount = 0;
  }
}  
void getMessage(){
   if (gw.messageAvailable()) {
    message_s message = gw.getMessage();
    setRelayStatus(message);
  }
}
//
void setRelayStatus(message_s message) {
  if (message.header.messageType=M_SET_VARIABLE &&
      message.header.type==V_LIGHT) {
     incomingRelayStatus = atoi(message.data);
     // Change relay state
     digitalWrite(message.header.childId, incomingRelayStatus==1?RELAY_ON:RELAY_OFF);
     // Write some debug info
     Serial.print("Incoming change for relay on pin:");
     Serial.print(message.header.childId);
     Serial.print(", New status: ");
     Serial.println(incomingRelayStatus);
   }
}
void pwmWrite() {
  if (ledState==HIGH){
        analogWrite(led,analogLevel);
      }
      else{
        digitalWrite(led,LOW);
      }
}
